package view;

import javafx.scene.Group;
import javafx.scene.shape.Box;
import javafx.scene.shape.DrawMode;
import javafx.scene.shape.MeshView;
import javafx.scene.shape.TriangleMesh;
import model.Piece;
import model.Position;

public class BoardView {

	public static final int BOARD_SIZE = 16, CENTERED_ORIGIN = -BOARD_SIZE*7/2;

	private Box[] board; // contains all the squares of the board (visual)
	private MeshView[] pieces; // contains all the 3d pieces (visual)
	public BoardView(Custom3dModel model, CustomMaterial material) {
		// creates the board squares, and position them properly
		board = new Box[64];
		for(int j = 0 ; j < 8 ; j++) {
			for (int i = 0; i < 8; i++) {
				int n = j * 8 + i;
				board[n] = new Box(BOARD_SIZE, BOARD_SIZE / 2, BOARD_SIZE);
				board[n].translateXProperty().set(i * BOARD_SIZE + CENTERED_ORIGIN);
				board[n].translateZProperty().set(j * BOARD_SIZE + CENTERED_ORIGIN);
			}
		}
		resetColor(material); // init/reset the board squares with black and white

		// creates the 3d pieces
		pieces = new MeshView[64];
		for(int i = 0 ; i < 64 ; i++) {
			pieces[i] = null;
		}
		int y;
		for(int color = Piece.WHITE; color == Piece.WHITE || color == Piece.BLACK ; color++) {
			// placement of pawns
			if(color == Piece.WHITE) { y = 2; }
			else { y = 7; }
			for(int x = Position.a ; x <= Position.h ; x++) {
				createPiece(x, y, model.get(Custom3dModel.PAWN));
			}
			// placement of rook, knight, bishop, queen, king
			if(color == Piece.WHITE) { y = 1; }
			else { y = 8; }
			createPiece(Position.a, y, model.get(Custom3dModel.ROOK));
			createPiece(Position.b, y, model.get(Custom3dModel.KNIGHT));
			createPiece(Position.c, y, model.get(Custom3dModel.BISHOP));
			createPiece(Position.d, y, model.get(Custom3dModel.QUEEN));
			createPiece(Position.e, y, model.get(Custom3dModel.KING));
			createPiece(Position.f, y, model.get(Custom3dModel.BISHOP));
			createPiece(Position.g, y, model.get(Custom3dModel.KNIGHT));
			createPiece(Position.h, y, model.get(Custom3dModel.ROOK));
		}
		// init materials, drawmode, and scale, for all pieces
		int scale = 16;
		for(int j = 0 ; j < 8 ; j++) {
			for (int i = 0; i < 8; i++) {
				int n = j*8+i;
				if(pieces[n] != null) {
					pieces[n].setDrawMode(DrawMode.FILL);
					if(j < 4) // distinction between white and black player
						pieces[n].setMaterial(material.get(CustomMaterial.WHITE));
					else
						pieces[n].setMaterial(material.get(CustomMaterial.BLACK));
					pieces[n].setScaleX(scale);
					pieces[n].setScaleY(-scale); // Y is down (not up) in javaFX, so we need to reverse object
					pieces[n].setScaleZ(scale);
					pieces[n].translateYProperty().set((float)-12); // place at right height
				}
			}
		}
		// updates the board-position for all the pieces to their right place
		// (we need to, because by default they are all at center of board)
		updateAllPieces();
	}

	public void updateAllPieces() { // updates all the pieces, visually, to the right position
		for(int j = 1 ; j <= 8 ; j++) {
			for (int i = 1; i <= 8; i++) {
				updatePiece(i, j);
			}
		}
	}

	public void updatePiece(int i, int j) { // updates the piece(i, j), visually, to the right position
		Position p = new Position(i ,j);
		if(pieces[p.getID()] != null) {
			pieces[p.getID()].translateXProperty().set((float) ((i-1) * BOARD_SIZE + CENTERED_ORIGIN));
			pieces[p.getID()].translateZProperty().set((float) ((j-1) * BOARD_SIZE + CENTERED_ORIGIN));
		}
	}

	private void createPiece(int x, int y, TriangleMesh mesh) {
		pieces[new Position(x,y).getID()] = new MeshView(mesh);
	}

	public Box[] getBoard() {
		return board;
	}

	public MeshView[] getPieces() {
		return pieces;
	}

	public void resetColor(CustomMaterial material) {
		for(int j = 0 ; j < 8 ; j++) {
			for (int i = 0; i < 8; i++) {
				int n = j * 8 + i;
				// resets board with black and white - bottom left (first case) is black, then alternates
				if((i + j) % 2 == 0) {
					board[n].setMaterial(material.get(CustomMaterial.BOARDBLACK));
				}
				else {
					board[n].setMaterial(material.get(CustomMaterial.BOARDWHITE));
				}
			}
		}
	}

	public void setColor(CustomMaterial material, Position position, int type) {
		board[position.getID()].setMaterial(material.get(type));
	}

	public void addAllToScene(Group group) {
		for(int n = 0 ; n < 64 ; n++) {
			if(board[n] != null) {
				group.getChildren().addAll(board[n]);
			}
			if(pieces[n] != null) {
				group.getChildren().addAll(pieces[n]);
			}
		}
	}

	// board click event, PAS TOUCHER POUR LINSTANT, A RE-UTILISER POUR EVENT DES PIECES
	/*board[n].addEventHandler(MouseEvent.MOUSE_CLICKED, event -> {
		for (int j2 = 0; j2 < 8; j2++) {
			for (int i2 = 0; i2 < 8; i2++) {
				int n2 = j2 * 8 + i2;
				board[n2].setMaterial(material.get((i2 + j2) % 2));
			}
		}
		board[n].setMaterial(material.get(CustomMaterial.SELECTED));
	});*/
}
